@@ -7,7 +7,7 @@ use Carp;
 require Exporter;
 require DynaLoader;
 
-our $VERSION = 1.71;
+our $VERSION = 1.8;
 
 our @ISA = qw(Exporter DynaLoader);
 
@@ -89,7 +89,7 @@ __END__
 
 =head1 NAME
 
-Convert::UUlib - Perl interface to the uulib library (a.k.a. uudeview/uuenview).
+Convert::UUlib - decode uu/xx/b64/mime/yenc/etc-encoded data from a massive number of files
 
 =head1 SYNOPSIS
 
@@ -108,6 +108,16 @@ Convert::UUlib - Perl interface to the uulib library (a.k.a. uudeview/uuenview).
 
 =head1 DESCRIPTION
 
+This module started as an interface to the uulib/uudeview library by Frank
+Pilhofer that can be used to decode all kinds of usenet (and other)
+binary messages.
+
+After upstream abondoned the project, th library was continuously bugfixed
+and improved in this module, with major focuses on security fixes,
+correctness and speed (that does not mean that this library is considered
+safe with untrusted data, but it surely is safer than the poriginal
+uudeview).
+
 Read the file doc/library.pdf from the distribution for in-depth
 information about the C-library used in this interface, and the rest of
 this document and especially the non-trivial decoder program at the end.
@@ -294,7 +304,9 @@ C<GetFileList>.
 
 =item @items = GetFileList
 
-Similar to C<GetFileListItem>, but returns all files in one go.
+Similar to C<GetFileListItem>, but returns all files in one go, which is
+very much faster for large number of items, and has no drawbacks when used
+for a small number of items.
 
 =back
 
@@ -484,6 +496,7 @@ illustrates the above worklfow in a non-trivial example.
    SetOption OPT_IGNMODE, 1;
    SetOption OPT_IGNMODE, 1;
    SetOption OPT_VERBOSE, 1;
+   SetOption OPT_AUTOCHK, 0;
 
    # show the three ways you can set callback functions. I normally
    # prefer the one with the sub inplace.
@@ -532,6 +545,8 @@ illustrates the above worklfow in a non-trivial example.
       print "file($_), status(", strerror $retval, ") parts($count)\n";
    }
 
+   Smerge -1;
+
    SetOption OPT_SAVEPATH, "uudst/";
 
    # now wade through all files and their source parts
