@@ -69,7 +69,7 @@
 
 #define UUPERCENT(a,b)	((int) ((unsigned long)(a) / \
 				(((unsigned long)(b)/100)+1)))
-     
+
 /*
  * Make the Busy Callback easier. The macro returns true if the BusyCallback
  * wants us to terminate.
@@ -142,14 +142,15 @@ typedef struct _fileread {
   char *origin;		/* Whole 'From:' line */
   char *mimeid;		/* the ID for Mime-encoded files */
   char *mimetype;	/* Content-Type */
-  short mode;		/* Mode of File (from 'begin') */
   int   begin;		/* begin detected */
   int   end;		/* end detected */
-  int   flags;		/* associated flags */
+  long  yefilesize;     /* the yencode file size, or 0 */
 
-  short uudet;		/* valid encoded data. value indicates encoding */
-  short partno;		/* Mime-files have a part number within */
-  short maxpno;		/* ... plus the total number of parts   */
+  uint16_t mode;	/* Mode of File (from 'begin') */
+  uint8_t flags;	/* associated flags */
+  uint8_t uudet;	/* valid encoded data. value indicates encoding */
+  int   partno;		/* Mime-files have a part number within */
+  int   maxpno;		/* ... plus the total number of parts   */
 
   char *sfname;		/* Associated source file */
   long startpos;	/* ftell() position where data starts */
@@ -164,13 +165,14 @@ typedef struct _fileread {
  **/
 
 typedef struct _uufile {
+  struct _uufile *NEXT;
   char     *filename;
   char     *subfname;
   char     *mimeid;
   char     *mimetype;
-  short     partno;
   fileread *data;
-  struct _uufile *NEXT;
+  long      yefilesize;
+  int       partno;
 } uufile;
 
 extern void *uu_MsgCBArg;
@@ -183,6 +185,12 @@ extern void *uu_FNCBArg;
  * variables
  */
 
+extern int uu_autocheck;
+extern int uu_rbuf;
+extern int uu_wbuf;
+#define UUSETBUF(fp,buff,size) if (size) setvbuf ((fp), ((buff) = malloc (size)), _IOFBF, (size))
+#define UUCLRBUF(size,buff) if (size) free (buff)
+
 extern int uu_fast_scanning;
 extern int uu_bracket_policy;
 extern int uu_verbose;
@@ -280,7 +288,6 @@ int		UUBusyPoll		(void);
 
 uufile *	UUPreProcessPart	(fileread *, int *);
 int 		UUInsertPartToList	(uufile *);
-uulist *	UUCheckGlobalList	(void);
 
 /*
  * Functions from uuutil.c
@@ -295,7 +302,7 @@ fileread *	ScanPart 		(FILE *, char *, int *);
 
 int		UUbhdecomp		(char *, char *,
 				         char *, int *,
-				         size_t, size_t, 
+				         size_t, size_t,
 				         size_t *);
 size_t		UUbhwrite		(char *, size_t, size_t, FILE *);
 
