@@ -81,10 +81,12 @@
 #include <fptools.h>
 #include <uustring.h>
 
-char * uulib_id = "$Id: uulib.c,v 1.10 2002/04/06 02:28:35 root Exp $";
+#include "safealloc.c"
+
+char * uulib_id = "$Id$";
 
 #ifdef SYSTEM_WINDLL
-BOOL _export WINAPI 
+BOOL _export WINAPI
 DllEntryPoint (HINSTANCE hInstance, DWORD seginfo,
 	       LPVOID lpCmdLine)
 {
@@ -126,6 +128,12 @@ void *uu_FileCBArg = NULL;
 void *uu_FFCBArg   = NULL;
 void *uu_FNCBArg;
 
+int uu_autocheck = 1;           /* call UUCheckGlobaList after every part */
+
+/* I/O buffer sizes */
+int uu_rbuf = 0;
+int uu_wbuf = 0;
+
 /*
  * Global variables
  */
@@ -236,7 +244,7 @@ static allomap toallocate[] = {
   { &uuestr_itemp,       256 },  /* from uuencode.c:UUEncodeStream() */
   { &uuestr_otemp,      1024 },
   { &uulib_msgstring,   1024 },  /* from uulib.c:UUMessage() */
-  { &uuncdl_fulline,    1200 },  /* from uunconc.c:UUDecodeLine() */
+  { &uuncdl_fulline,    1201 },  /* from uunconc.c:UUDecodeLine() */
   { &uuncdp_oline,      3600 },  /* from uunconc.c:UUDecodePart() */
   { &uunconc_UUxlat,     256 * sizeof (int) },  /* from uunconc.c:toplevel */
   { &uunconc_UUxlen,      64 * sizeof (int) },
@@ -245,12 +253,13 @@ static allomap toallocate[] = {
   { &uunconc_BHxlat,     256 * sizeof (int) },
   { &uunconc_save,    3*1200 },  /* from uunconc.c:decoding buffer */
   { &uuscan_shlline,    1024 },  /* from uuscan.c:ScanHeaderLine() */
+  { &uuscan_shlline2,   1024 },  /* from uuscan.c:ScanHeaderLine() */
   { &uuscan_pvvalue,     300 },  /* from uuscan.c:ParseValue() */
   { &uuscan_phtext,      300 },  /* from uuscan.c:ParseHeader() */
-  { &uuscan_sdline,      300 },  /* from uuscan.c:ScanData() */
-  { &uuscan_sdbhds1,     300 },
-  { &uuscan_sdbhds2,     300 },
-  { &uuscan_spline,      300 },  /* from uuscan.c:ScanPart() */
+  { &uuscan_sdline,     1025 },  /* from uuscan.c:ScanData(), +1 for UURepairData */
+  { &uuscan_sdbhds1,    1300 },  /* 1024 for line in UUScan + 256 potential overhead UUDecodeLine */
+  { &uuscan_sdbhds2,    1300 },  /* 1024 for line in UUScan + 256 potential overhead UUDecodeLine */
+  { &uuscan_spline,     1024 },  /* from uuscan.c:ScanPart() */
   { &uuutil_bhwtmp,      300 },  /* from uuutil.c:UUbhwrite() */
   { NULL, 0 }
 };
@@ -268,7 +277,7 @@ UUMessage (va_alist)
   va_dcl
 #endif
 {
-  char *msgptr;
+  int msgofs;
 #if defined(STDC_HEADERS) || defined(HAVE_STDARG_H)
   va_list ap;
 
@@ -286,16 +295,16 @@ UUMessage (va_alist)
 #endif
 
   if (uu_debug) {
-    sprintf (uulib_msgstring, "%s(%d): %s", file, line, msgnames[level]);
-    msgptr = uulib_msgstring + strlen (uulib_msgstring);
+    snprintf (uulib_msgstring, 1024,  "%s(%d): %s", file, line, msgnames[level]);
+    msgofs = strlen (uulib_msgstring);
   }
   else {
-    sprintf (uulib_msgstring, "%s", msgnames[level]);
-    msgptr = uulib_msgstring + strlen (uulib_msgstring);
+    snprintf (uulib_msgstring, 1024, "%s", msgnames[level]);
+    msgofs = strlen (uulib_msgstring);
   }
 
   if (uu_MsgCallback && (level>UUMSG_NOTE || uu_verbose)) {
-    vsprintf (msgptr, format, ap);
+    vsnprintf (uulib_msgstring + msgofs, 1024 - msgofs, format, ap);
 
     (*uu_MsgCallback) (uu_MsgCBArg, uulib_msgstring, level);
   }
@@ -389,13 +398,13 @@ UUInitialize (void)
     *(aiter->ptr) = NULL;
 
   for (aiter=toallocate; aiter->ptr; aiter++) {
-    if ((*(aiter->ptr) = (char *) malloc (aiter->size)) == NULL) {
+    if ((*(aiter->ptr) = (char *) safe_alloc (aiter->size)) == NULL) {
       /*
        * oops. we may not print a message here, because we need these
        * areas (uulib_msgstring) in UUMessage()
        */
       for (aiter=toallocate; aiter->ptr; aiter++) {
-	_FP_free (*(aiter->ptr));
+	safe_free (*(aiter->ptr), aiter->size);
       }
       return UURET_NOMEM;
     }
@@ -420,8 +429,16 @@ UUGetOption (int option, int *ivalue, char *cvalue, int clength)
   int result;
 
   switch (option) {
+  case UUOPT_RBUF:
+    *ivalue = uu_rbuf;
+    result = 0;
+    break;
+  case UUOPT_WBUF:
+    *ivalue = uu_wbuf;
+    result = 0;
+    break;
   case UUOPT_VERSION:
-    _FP_strncpy (cvalue, uulibversion, clength);
+    FP_strncpy (cvalue, uulibversion, clength);
     result = 0;
     break;
   case UUOPT_FAST:
@@ -461,7 +478,7 @@ UUGetOption (int option, int *ivalue, char *cvalue, int clength)
     result = uu_overwrite;
     break;
   case UUOPT_SAVEPATH:
-    _FP_strncpy (cvalue, uusavepath, clength);
+    FP_strncpy (cvalue, uusavepath, clength);
     result = 0;
     break;
   case UUOPT_PROGRESS:
@@ -489,7 +506,7 @@ UUGetOption (int option, int *ivalue, char *cvalue, int clength)
     result = uu_tinyb64;
     break;
   case UUOPT_ENCEXT:
-    _FP_strncpy (cvalue, uuencodeext, clength);
+    FP_strncpy (cvalue, uuencodeext, clength);
     result = 0;
     break;
   case UUOPT_REMOVE:
@@ -504,6 +521,10 @@ UUGetOption (int option, int *ivalue, char *cvalue, int clength)
     if (ivalue) *ivalue = uu_dotdot;
     result = uu_dotdot;
     break;
+  case UUOPT_AUTOCHECK:
+    if (ivalue) *ivalue = uu_autocheck;
+    result = uu_autocheck;
+    break;
   default:
     return -1;
   }
@@ -514,6 +535,12 @@ int UUEXPORT
 UUSetOption (int option, int ivalue, char *cvalue)
 {
   switch (option) {
+  case UUOPT_RBUF:
+    uu_rbuf = ivalue;
+    break;
+  case UUOPT_WBUF:
+    uu_wbuf = ivalue;
+    break;
   case UUOPT_FAST:
     uu_fast_scanning  = ivalue;
     break;
@@ -539,8 +566,8 @@ UUSetOption (int option, int ivalue, char *cvalue)
     uu_overwrite      = ivalue;
     break;
   case UUOPT_SAVEPATH:
-    _FP_free (uusavepath);
-    uusavepath = _FP_strdup (cvalue);
+    FP_free (uusavepath);
+    uusavepath = FP_strdup (cvalue);
     break;
   case UUOPT_IGNMODE:
     uu_ignmode = ivalue;
@@ -555,8 +582,8 @@ UUSetOption (int option, int ivalue, char *cvalue)
     uu_tinyb64 = ivalue;
     break;
   case UUOPT_ENCEXT:
-    _FP_free (uuencodeext);
-    uuencodeext = _FP_strdup (cvalue);
+    FP_free (uuencodeext);
+    uuencodeext = FP_strdup (cvalue);
     break;
   case UUOPT_REMOVE:
     uu_remove_input = ivalue;
@@ -567,6 +594,9 @@ UUSetOption (int option, int ivalue, char *cvalue)
   case UUOPT_DOTDOT:
     uu_dotdot = ivalue;
     break;
+  case UUOPT_AUTOCHECK:
+    uu_autocheck = ivalue;
+    break;
   default:
     return UURET_ILLVAL;
   }
@@ -584,7 +614,7 @@ UUstrerror (int code)
  */
 
 int UUEXPORT
-UUSetMsgCallback (void *opaque, 
+UUSetMsgCallback (void *opaque,
 		  void (*func) (void *, char *, int))
 {
   uu_MsgCallback = func;
@@ -674,6 +704,12 @@ UUFNameFilter (char *fname)
 
 int UUEXPORT
 UULoadFile (char *filename, char *fileid, int delflag, int *partcount)
+{
+  return UULoadFileWithPartNo(filename, fileid, delflag, -1, partcount);
+}
+
+int UUEXPORT
+UULoadFileWithPartNo (char *filename, char *fileid, int delflag, int partno, int *partcount)
 {
   int res, sr;
   struct stat finfo;
@@ -681,11 +717,12 @@ UULoadFile (char *filename, char *fileid, int delflag, int *partcount)
   uufile *fload;
   itbd *killem;
   FILE *datei;
+  void *datei_buf;
 
   int _count;
   if (!partcount)
     partcount = &_count;
-  
+
   *partcount = 0;
 
   if ((datei = fopen (filename, "rb")) == NULL) {
@@ -694,12 +731,15 @@ UULoadFile (char *filename, char *fileid, int delflag, int *partcount)
 	       filename, strerror (uu_errno = errno));
     return UURET_IOERR;
   }
+  UUSETBUF (datei, datei_buf, uu_rbuf);
+  FP_flockfile (datei);
 
   if (fstat (fileno(datei), &finfo) == -1) {
     UUMessage (uulib_id, __LINE__, UUMSG_ERROR,
 	       uustring (S_NOT_STAT_FILE),
 	       filename, strerror (uu_errno = errno));
     fclose (datei);
+    UUCLRBUF (uu_rbuf, datei_buf);
     return UURET_IOERR;
   }
 
@@ -712,10 +752,10 @@ UULoadFile (char *filename, char *fileid, int delflag, int *partcount)
       UUMessage (uulib_id, __LINE__, UUMSG_WARNING,
 		 uustring (S_OUT_OF_MEMORY), sizeof (itbd));
     }
-    else if ((killem->fname = _FP_strdup (filename)) == NULL) {
+    else if ((killem->fname = FP_strdup (filename)) == NULL) {
       UUMessage (uulib_id, __LINE__, UUMSG_WARNING,
 		 uustring (S_OUT_OF_MEMORY), strlen(filename)+1);
-      _FP_free (killem);
+      FP_free (killem);
     }
     else {
       killem->NEXT = ftodel;
@@ -729,7 +769,7 @@ UULoadFile (char *filename, char *fileid, int delflag, int *partcount)
   progress.fsize    = (long) ((finfo.st_size>0)?finfo.st_size:-1);
   progress.percent  = 0;
   progress.foffset  = 0;
-  _FP_strncpy (progress.curfile,
+  FP_strncpy (progress.curfile,
 	       (strlen(filename)>255)?
 	       (filename+strlen(filename)-255):filename,
 	       256);
@@ -738,16 +778,16 @@ UULoadFile (char *filename, char *fileid, int delflag, int *partcount)
   if (fileid == NULL)
     fileid = filename;
 
-  while (!feof (datei) && !ferror (datei)) {
-    /* 
+  while (!FP_feof (datei) && !ferror (datei)) {
+    /*
      * Peek file, or some systems won't detect EOF
      */
-    res = fgetc (datei);
-    if (feof (datei) || ferror (datei))
+    res = FP_getc (datei);
+    if (FP_feof (datei) || ferror (datei))
       break;
     else
       ungetc (res, datei);
-    
+
     if ((loaded = ScanPart (datei, fileid, &sr)) == NULL) {
       if (sr != UURET_NODATA && sr != UURET_OK && sr != UURET_CONT) {
 	UUkillfread (loaded);
@@ -756,9 +796,10 @@ UULoadFile (char *filename, char *fileid, int delflag, int *partcount)
 		     uustring (S_READ_ERROR), filename,
 		     strerror (uu_errno));
 
-	UUCheckGlobalList ();
+	if (uu_autocheck) UUCheckGlobalList ();
 	progress.action = 0;
 	fclose (datei);
+        UUCLRBUF (uu_rbuf, datei_buf);
 	return sr;
       }
       continue;
@@ -768,12 +809,16 @@ UULoadFile (char *filename, char *fileid, int delflag, int *partcount)
       UUMessage (uulib_id, __LINE__, UUMSG_ERROR,
 		 uustring (S_READ_ERROR), filename,
 		 strerror (uu_errno = errno));
-      UUCheckGlobalList ();
+      if (uu_autocheck) UUCheckGlobalList ();
       progress.action = 0;
       fclose (datei);
+      UUCLRBUF (uu_rbuf, datei_buf);
       return UURET_IOERR;
     }
 
+    if (partno != -1)
+      loaded->partno = partno;
+
     if ((loaded->uudet == QP_ENCODED || loaded->uudet == PT_ENCODED) &&
 	(loaded->filename == NULL || *(loaded->filename) == '\0') &&
 	!uu_handletext && (loaded->flags&FL_PARTIAL)==0) {
@@ -795,7 +840,7 @@ UULoadFile (char *filename, char *fileid, int delflag, int *partcount)
       if (uu_fast_scanning && sr != UURET_CONT) break;
       continue;
     }
-    
+
     if ((fload = UUPreProcessPart (loaded, &res)) == NULL) {
       /*
        * no useful data found
@@ -825,7 +870,7 @@ UULoadFile (char *filename, char *fileid, int delflag, int *partcount)
 		 (loaded->end)      ? "end"   : "",
 		 codenames[loaded->uudet]);
     }
-    
+
     if ((res = UUInsertPartToList (fload))) {
       /*
        * couldn't use the data
@@ -833,9 +878,10 @@ UULoadFile (char *filename, char *fileid, int delflag, int *partcount)
       UUkillfile (fload);
 
       if (res != UURET_NODATA) {
-	UUCheckGlobalList ();
+	if (uu_autocheck) UUCheckGlobalList ();
 	progress.action = 0;
 	fclose (datei);
+        UUCLRBUF (uu_rbuf, datei_buf);
 	return res;
       }
       if (uu_fast_scanning && sr != UURET_CONT)
@@ -855,14 +901,27 @@ UULoadFile (char *filename, char *fileid, int delflag, int *partcount)
     if (uu_fast_scanning && sr != UURET_CONT)
       break;
   }
+
+  if (ferror (datei)) {
+    UUMessage (uulib_id, __LINE__, UUMSG_ERROR,
+	       uustring (S_READ_ERROR), filename,
+	       strerror (uu_errno = errno));
+    if (uu_autocheck) UUCheckGlobalList ();
+    progress.action = 0;
+    fclose (datei);
+    UUCLRBUF (uu_rbuf, datei_buf);
+    return UURET_IOERR;
+  }
+
   fclose (datei);
+  UUCLRBUF (uu_rbuf, datei_buf);
 
   if (!uu_fast_scanning && *partcount == 0)
     UUMessage (uulib_id, __LINE__, UUMSG_NOTE,
 	       uustring (S_NO_DATA_FOUND), filename);
 
   progress.action = 0;
-  UUCheckGlobalList ();
+  if (uu_autocheck) UUCheckGlobalList ();
 
   return UURET_OK;
 }
@@ -885,6 +944,7 @@ int UUEXPORT
 UUDecodeFile (uulist *thefile, char *destname)
 {
   FILE *target, *source;
+  void *target_buf, *source_buf;
   struct stat finfo;
   int fildes, res;
   size_t bytes;
@@ -908,6 +968,8 @@ UUDecodeFile (uulist *thefile, char *destname)
 	       thefile->binfile, strerror (uu_errno = errno));
     return UURET_IOERR;
   }
+  UUSETBUF (source, source_buf, uu_rbuf);
+  FP_flockfile (source);
 
   /*
    * for system security, strip setuid/setgid bits from mode
@@ -922,7 +984,7 @@ UUDecodeFile (uulist *thefile, char *destname)
 
   /*
    * Determine the name of the target file according to the rules:
-   * 
+   *
    * IF (destname!=NULL) THEN filename=destname;
    * ELSE
    *   filename = thefile->filename
@@ -934,10 +996,10 @@ UUDecodeFile (uulist *thefile, char *destname)
   if (destname)
     strcpy (uugen_fnbuffer, destname);
   else {
+    char *fname = UUFNameFilter (thefile->filename ? thefile->filename : "unknown.xxx");
     sprintf (uugen_fnbuffer, "%.1024s%.3071s",
-	     (uusavepath)?uusavepath:"",
-	     UUFNameFilter ((thefile->filename)?
-			    thefile->filename:"unknown.xxx"));
+	     uusavepath ? uusavepath : "",
+             fname ? fname : "unknown.xxx");
   }
 
   /*
@@ -949,6 +1011,7 @@ UUDecodeFile (uulist *thefile, char *destname)
       UUMessage (uulib_id, __LINE__, UUMSG_ERROR,
 		 uustring (S_TARGET_EXISTS), uugen_fnbuffer);
       fclose (source);
+      UUCLRBUF (uu_rbuf, source_buf);
       return UURET_EXISTS;
     }
   }
@@ -958,6 +1021,7 @@ UUDecodeFile (uulist *thefile, char *destname)
 	       uustring (S_NOT_STAT_FILE),
 	       thefile->binfile, strerror (uu_errno = errno));
     fclose (source);
+    UUCLRBUF (uu_rbuf, source_buf);
     return UURET_IOERR;
   }
 
@@ -969,6 +1033,7 @@ UUDecodeFile (uulist *thefile, char *destname)
       mask = umask (0022); umask (mask);
 #endif
       fclose (source);
+      UUCLRBUF (uu_rbuf, source_buf);
 #if HAVE_CHMOD
       chmod (uugen_fnbuffer, thefile->mode & ~mask);
 #endif
@@ -976,7 +1041,7 @@ UUDecodeFile (uulist *thefile, char *destname)
     }
 
   progress.action   = 0;
-  _FP_strncpy (progress.curfile,
+  FP_strncpy (progress.curfile,
 	       (strlen(uugen_fnbuffer)>255)?
 	       (uugen_fnbuffer+strlen(uugen_fnbuffer)-255):uugen_fnbuffer,
 	       256);
@@ -995,6 +1060,7 @@ UUDecodeFile (uulist *thefile, char *destname)
 	       uustring (S_NOT_OPEN_TARGET),
 	       uugen_fnbuffer, strerror (uu_errno = errno));
     fclose (source);
+    UUCLRBUF (uu_rbuf, source_buf);
     return UURET_IOERR;
   }
 
@@ -1004,30 +1070,37 @@ UUDecodeFile (uulist *thefile, char *destname)
 	       uustring (S_IO_ERR_TARGET),
 	       uugen_fnbuffer, strerror (uu_errno = errno));
     fclose (source);
+    UUCLRBUF (uu_rbuf, source_buf);
     close  (fildes);
     return UURET_IOERR;
   }
+  UUSETBUF (target, target_buf, uu_wbuf);
+  FP_flockfile (target);
 
-  while (!feof (source)) {
+  while (!FP_feof (source)) {
 
     if (UUBUSYPOLL(ftell(source),progress.fsize)) {
       UUMessage (uulib_id, __LINE__, UUMSG_NOTE,
 		 uustring (S_DECODE_CANCEL));
       fclose (source);
+      UUCLRBUF (uu_rbuf, source_buf);
       fclose (target);
+      UUCLRBUF (uu_wbuf, target_buf);
       unlink (uugen_fnbuffer);
       return UURET_CANCEL;
     }
 
     bytes = fread (uugen_inbuffer, 1, 1024, source);
 
-    if (ferror (source) || (bytes == 0 && !feof (source))) {
+    if (ferror (source) || (bytes == 0 && !FP_feof (source))) {
       progress.action = 0;
       UUMessage (uulib_id, __LINE__, UUMSG_ERROR,
 		 uustring (S_READ_ERROR),
 		 thefile->binfile, strerror (uu_errno = errno));
       fclose (source);
+      UUCLRBUF (uu_rbuf, source_buf);
       fclose (target);
+      UUCLRBUF (uu_wbuf, target_buf);
       unlink (uugen_fnbuffer);
       return UURET_IOERR;
     }
@@ -1037,14 +1110,25 @@ UUDecodeFile (uulist *thefile, char *destname)
 		 uustring (S_WR_ERR_TARGET),
 		 uugen_fnbuffer, strerror (uu_errno = errno));
       fclose (source);
+      UUCLRBUF (uu_rbuf, source_buf);
       fclose (target);
+      UUCLRBUF (uu_wbuf, target_buf);
       unlink (uugen_fnbuffer);
       return UURET_IOERR;
     }
   }
 
-  fclose (target);
   fclose (source);
+  UUCLRBUF (uu_rbuf, source_buf);
+  if (fclose (target)) {
+    UUCLRBUF (uu_wbuf, target_buf);
+    UUMessage (uulib_id, __LINE__, UUMSG_ERROR,
+	       uustring (S_WR_ERR_TARGET),
+	       uugen_fnbuffer, strerror (uu_errno = errno));
+    unlink (uugen_fnbuffer);
+    return UURET_IOERR;
+  }
+  UUCLRBUF (uu_wbuf, target_buf);
 
   /*
    * after a successful decoding run, we delete the temporary file
@@ -1058,7 +1142,7 @@ UUDecodeFile (uulist *thefile, char *destname)
   }
 
 skip_copy:
-  _FP_free (thefile->binfile);
+  FP_free (thefile->binfile);
   thefile->binfile = NULL;
   thefile->state  &= ~UUFILE_TMPFILE;
   thefile->state  |=  UUFILE_DECODED;
@@ -1085,7 +1169,7 @@ UUInfoFile (uulist *thefile, void *opaque,
    */
 
   if (uu_FileCallback) {
-    if ((res = (*uu_FileCallback) (uu_FileCBArg, 
+    if ((res = (*uu_FileCallback) (uu_FileCBArg,
 				   thefile->thisfile->data->sfname,
 				   uugen_fnbuffer,
 				   1)) != UURET_OK)
@@ -1102,12 +1186,12 @@ UUInfoFile (uulist *thefile, void *opaque,
   else {
     if ((inpfile = fopen (thefile->thisfile->data->sfname, "rb")) == NULL) {
       UUMessage (uulib_id, __LINE__, UUMSG_ERROR,
-		 uustring (S_NOT_OPEN_FILE), 
+		 uustring (S_NOT_OPEN_FILE),
 		 thefile->thisfile->data->sfname,
 		 strerror (uu_errno=errno));
       return UURET_IOERR;
     }
-    _FP_strncpy (uugen_fnbuffer, thefile->thisfile->data->sfname, 1024);
+    FP_strncpy (uugen_fnbuffer, thefile->thisfile->data->sfname, 1024);
   }
 
   /*
@@ -1117,9 +1201,9 @@ UUInfoFile (uulist *thefile, void *opaque,
   fseek (inpfile, thefile->thisfile->data->startpos, SEEK_SET);
   maxpos = thefile->thisfile->data->startpos + thefile->thisfile->data->length;
 
-  while (!feof (inpfile) && 
+  while (!FP_feof (inpfile) &&
 	 (uu_fast_scanning || ftell(inpfile) < maxpos)) {
-    if (_FP_fgets (uugen_inbuffer, 511, inpfile) == NULL)
+    if (FP_fgets (uugen_inbuffer, 511, inpfile) == NULL)
       break;
     uugen_inbuffer[511] = '\0';
 
@@ -1135,6 +1219,9 @@ UUInfoFile (uulist *thefile, void *opaque,
     else if ((thefile->uudet == UU_ENCODED || thefile->uudet == XX_ENCODED) &&
 	     strncmp (uugen_inbuffer, "begin ", 6) == 0)
       break;
+    else if (thefile->uudet == YENC_ENCODED &&
+	     strncmp (uugen_inbuffer, "=ybegin ", 8) == 0)
+      break;
 
     if ((*func) (opaque, uugen_inbuffer))
       break;
@@ -1150,7 +1237,7 @@ UUInfoFile (uulist *thefile, void *opaque,
   fclose (inpfile);
 
   if (uu_FileCallback)
-    (*uu_FileCallback) (uu_FileCBArg, 
+    (*uu_FileCallback) (uu_FileCBArg,
 			thefile->thisfile->data->sfname,
 			uugen_fnbuffer, 0);
 
@@ -1159,7 +1246,7 @@ UUInfoFile (uulist *thefile, void *opaque,
 
   return UURET_OK;
 }
-	    
+
 int UUEXPORT
 UURenameFile (uulist *thefile, char *newname)
 {
@@ -1170,14 +1257,14 @@ UURenameFile (uulist *thefile, char *newname)
 
   oldname = thefile->filename;
 
-  if ((thefile->filename = _FP_strdup (newname)) == NULL) {
+  if ((thefile->filename = FP_strdup (newname)) == NULL) {
     UUMessage (uulib_id, __LINE__, UUMSG_ERROR,
 	       uustring (S_NOT_RENAME),
 	       oldname, newname);
     thefile->filename = oldname;
     return UURET_NOMEM;
   }
-  _FP_free (oldname);
+  FP_free (oldname);
   return UURET_OK;
 }
 
@@ -1194,7 +1281,7 @@ UURemoveTemp (uulist *thefile)
 		 thefile->binfile,
 		 strerror (uu_errno = errno));
     }
-    _FP_free (thefile->binfile);
+    FP_free (thefile->binfile);
     thefile->binfile = NULL;
     thefile->state  &= ~UUFILE_TMPFILE;
   }
@@ -1219,10 +1306,10 @@ UUCleanUp (void)
 		 uustring (S_TMP_NOT_REMOVED),
 		 iter->fname, strerror (uu_errno = errno));
     }
-    _FP_free (iter->fname);
+    FP_free (iter->fname);
     ptr  = iter;
     iter = iter->NEXT;
-    _FP_free (ptr);
+    FP_free (ptr);
   }
 
   ftodel = NULL;
@@ -1254,9 +1341,9 @@ UUCleanUp (void)
   UUkilllist (UUGlobalFileList);
   UUGlobalFileList = NULL;
 
-  _FP_free (uusavepath);
-  _FP_free (uuencodeext);
-  _FP_free (sstate.source);
+  FP_free (uusavepath);
+  FP_free (uuencodeext);
+  FP_free (sstate.source);
 
   uusavepath  = NULL;
   uuencodeext = NULL;
@@ -1269,7 +1356,7 @@ UUCleanUp (void)
   while (mssdepth) {
     mssdepth--;
     UUkillheaders (&(multistack[mssdepth].envelope));
-    _FP_free (multistack[mssdepth].source);
+    FP_free (multistack[mssdepth].source);
   }
 
   /*
@@ -1277,7 +1364,7 @@ UUCleanUp (void)
    */
 
   for (aiter=toallocate; aiter->ptr; aiter++) {
-    _FP_free (*(aiter->ptr));
+    safe_free (*(aiter->ptr), aiter->size);
     *(aiter->ptr) = NULL;
   }
 
