@@ -196,8 +196,8 @@ UUBrokenByNetscape (char *string)
   if (string==NULL || (len=strlen(string))<3)
     return 0;
 
-  if ((ptr = _FP_stristr (string, "<a href=")) != NULL) {
-    if (_FP_stristr (string, "</a>") > ptr)
+  if ((ptr = FP_stristr (string, "<a href=")) != NULL) {
+    if (FP_stristr (string, "</a>") > ptr)
       return 2;
   }
 
@@ -207,7 +207,7 @@ UUBrokenByNetscape (char *string)
   if (*--ptr == ' ') ptr--;
   ptr--;
 
-  if (_FP_strnicmp (ptr, "<a", 2) == 0)
+  if (FP_strnicmp (ptr, "<a", 2) == 0)
     return 1;
 
   return 0;
@@ -217,7 +217,7 @@ UUBrokenByNetscape (char *string)
  * Try to repair a Netscape-corrupted line of data.
  * This must only be called on corrupted lines, since non-Netscape
  * data may even _get_ corrupted by this procedure.
- * 
+ *
  * Some checks are included multiply to speed up the procedure. For
  * example: (*p1!='<' || strnicmp(p1,"</a>",4)). If the first expression
  * becomes true, the costly function isn't called :-)
@@ -241,9 +241,9 @@ UUNetscapeCollapse (char *string)
    */
   while (*p1) {
     if (*p1 == '&') {
-      if      (_FP_strnicmp (p1, "&amp;", 5) == 0) { p1+=5; *p2++='&'; res=1; }
-      else if (_FP_strnicmp (p1, "&lt;",  4) == 0) { p1+=4; *p2++='<'; res=1; }
-      else if (_FP_strnicmp (p1, "&gt;",  4) == 0) { p1+=4; *p2++='>'; res=1; }
+      if      (FP_strnicmp (p1, "&amp;", 5) == 0) { p1+=5; *p2++='&'; res=1; }
+      else if (FP_strnicmp (p1, "&lt;",  4) == 0) { p1+=4; *p2++='<'; res=1; }
+      else if (FP_strnicmp (p1, "&gt;",  4) == 0) { p1+=4; *p2++='>'; res=1; }
       else *p2++ = *p1++;
       res = 1;
     }
@@ -257,16 +257,16 @@ UUNetscapeCollapse (char *string)
 
   while (*p1) {
     if (*p1 == '<') {
-      if ((_FP_strnicmp (p1, "<ahref=", 7) == 0 ||
-	   _FP_strnicmp (p1, "<a href=",8) == 0) && 
-	  (_FP_strstr (p1, "</a>") != 0 || _FP_strstr (p1, "</A>") != 0)) {
+      if ((FP_strnicmp (p1, "<ahref=", 7) == 0 ||
+	   FP_strnicmp (p1, "<a href=",8) == 0) &&
+	  (FP_strstr (p1, "</a>") != 0 || FP_strstr (p1, "</A>") != 0)) {
 	while (*p1 && *p1!='>')        p1++;
 	if (*p1=='\0' || *(p1+1)!='<') return 0;
 	p1++;
-	while (*p1 && (*p1!='<' || _FP_strnicmp(p1,"</a>",4)!=0)) {
+	while (*p1 && (*p1!='<' || FP_strnicmp(p1,"</a>",4)!=0)) {
 	  *p2++ = *p1++;
 	}
-	if (_FP_strnicmp(p1,"</a>",4) != 0)
+	if (FP_strnicmp(p1,"</a>",4) != 0)
 	  return 0;
 	p1+=4;
 	res=1;
@@ -300,14 +300,10 @@ UUValidData (char *ptr, int encoding, int *bhflag)
     return 0;              /* bad string */
   }
 
-  while (*s) {
-    s++;
-    len++;
-    i++;
-  }
+  if (encoding == YENC_ENCODED)
+    return YENC_ENCODED;
 
-  if (i == 0)
-    return 0;
+  i = strlen (s);
 
   switch (encoding) {
   case UU_ENCODED:
@@ -318,8 +314,6 @@ UUValidData (char *ptr, int encoding, int *bhflag)
     goto _t_B64;
   case BH_ENCODED:
     goto _t_Binhex;
-  case YENC_ENCODED:
-    return YENC_ENCODED;
   }
 
  _t_Binhex:                 /* Binhex Test */
@@ -542,7 +536,7 @@ UURepairData (FILE *datei, char *line, int encoding, int *bhflag)
       if (strlen (line) > 250)
 	break;
       ptr = line + strlen (line);
-      if (_FP_fgets (ptr, 299-(ptr-line), datei) == NULL)
+      if (FP_fgets (ptr, 299-(ptr-line), datei) == NULL)
 	break;
     }
     else {			/* don't need next line to repair */
@@ -609,7 +603,25 @@ UUDecodeLine (char *s, char *d, int method)
    */
   z1 = z2 = z3 = z4 = 0;
 
-  if (method == UU_ENCODED || method == XX_ENCODED) {
+  if (method == YENC_ENCODED) {
+    while (*s) {
+      if (ecb_expect_false (*s == '=')) {
+	if (*++s != '\0') {
+	  d[count++] = (char) ((int) *s - 64 - 42);
+	  s++;
+	}
+      }
+      #if 0 /* FP_fgets never leaves CR or LF in the buffer, so skip this */
+      else if (ecb_expect_false (*s == '\n' || *s == '\r')) {
+	s++; /* ignore */
+      }
+      #endif
+      else {
+	d[count++] = (char) ((int) *s++ - 42);
+      }
+    }
+  }
+  else if (method == UU_ENCODED || method == XX_ENCODED) {
     if (method == UU_ENCODED)
       table = UUxlat;
     else
@@ -625,20 +637,20 @@ UUDecodeLine (char *s, char *d, int method)
 
       if(i-- > 0)
 	d[count++] = c;
-      
+
       cc <<= 4;
       c    = table[ACAST(*s++)];
       cc  |= (c >> 2);
-      
+
       if(i-- > 0)
 	d[count++] = cc;
-      
+
       c <<= 6;
       c |= table[ACAST(*s++)];
-      
+
       if(i-- > 0)
 	d[count++] = c;
-      
+
       j -= 4;
     }
   }
@@ -706,22 +718,6 @@ UUDecodeLine (char *s, char *d, int method)
     while (BHxlat[ACAST(*s)] != -1)
       uuncdl_fulline[leftover++] = *s++;
   }
-  else if (method == YENC_ENCODED) {
-    while (*s) {
-      if (*s == '=') {
-	if (*++s != '\0') {
-	  d[count++] = (char) ((int) *s - 64 - 42);
-	  s++;
-	}
-      }
-      else if (*s == '\n' || *s == '\r') {
-	s++; /* ignore */
-      }
-      else {
-	d[count++] = (char) ((int) *s++ - 42);
-      }
-    }
-  }
 
   return count;
 }
@@ -740,10 +736,10 @@ UUDecodeQP (FILE *datain, FILE *dataout, int *state,
 
   uulboundary = -1;
 
-  while (!feof (datain) && 
+  while (!FP_feof (datain) &&
 	 (ftell(datain)<maxpos || flags&FL_TOEND ||
 	  (!(flags&FL_PROPER) && uu_fast_scanning))) {
-    if (_FP_fgets (line, 1023, datain) == NULL)
+    if (FP_fgets (line, 1023, datain) == NULL)
       break;
     if (ferror (datain)) {
       UUMessage (uunconc_id, __LINE__, UUMSG_ERROR,
@@ -815,7 +811,7 @@ UUDecodeQP (FILE *datain, FILE *dataout, int *state,
      * So if the part ends here, don't print a line break"
      */
     /* something is broken here now, but it was broken before */
-    if (!feof (datain) && 
+    if (!FP_feof (datain) &&
 	(ftell(datain)<maxpos || flags&FL_TOEND ||
 	 (!(flags&FL_PROPER) && uu_fast_scanning)))
       fprintf (dataout, "%s\n", p1);
@@ -840,10 +836,10 @@ UUDecodePT (FILE *datain, FILE *dataout, int *state,
 
   uulboundary = -1;
 
-  while (!feof (datain) && 
+  while (!FP_feof (datain) &&
 	 (ftell(datain)<maxpos || flags&FL_TOEND ||
 	  (!(flags&FL_PROPER) && uu_fast_scanning))) {
-    if (_FP_fgets (line, 1023, datain) == NULL)
+    if (FP_fgets (line, 1023, datain) == NULL)
       break;
     if (ferror (datain)) {
       UUMessage (uunconc_id, __LINE__, UUMSG_ERROR,
@@ -961,15 +957,15 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
   int warning=0, vlc=0, lc[2], hadct=0;
   int tc=0, tf=0, vflag, haddata=0, haddh=0;
   long yefilesize=0, yepartends=0, yenotlastpart=0;
-  crc32_t yepartcrc=crc32(0L, Z_NULL, 0);
-  static crc32_t yefilecrc=0;
+  crc32_t yepartcrc=CRC32_INIT;
+  static crc32_t yefilecrc;
   static int bhflag=0;
   size_t count=0;
   size_t yepartsize=0;
   char *ptr;
 
   if (datain == NULL || dataout == NULL) {
-    yefilecrc = crc32(0L, Z_NULL, 0);
+    yefilecrc = CRC32_INIT;
     bhflag = 0;
     return UURET_OK;
   }
@@ -994,10 +990,10 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
     *state = BEGIN;
   }
 
-  while (!feof (datain) && *state != DONE && 
+  while (!FP_feof (datain) && *state != DONE &&
 	 (ftell(datain)<maxpos || flags&FL_TOEND || maxpos==-1 ||
 	  (!(flags&FL_PROPER) && uu_fast_scanning))) {
-    if (_FP_fgets ((line = uugen_fnbuffer), 1200 - 5, datain) == NULL)
+    if (FP_fgets ((line = uugen_fnbuffer), 1200 - 5, datain) == NULL)
       break;
 
     /* optionally skip .. */
@@ -1028,7 +1024,7 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
       vlc = 0;
       continue;
     }
-    
+
     /*
      * Busy Polls
      */
@@ -1061,7 +1057,7 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
 
     if ((flags&FL_PROPER) == 0) {
       if (strncmp (line, "BEGIN", 5) == 0 &&
-	  _FP_strstr  (line, "CUT HERE")  && !tf) { /* I hate these lines */
+	  FP_strstr  (line, "CUT HERE")  && !tf) { /* I hate these lines */
 	tc = tf = vlc = 0;
 	continue;
       }
@@ -1077,14 +1073,14 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
 	haddh = 1;
 	continue;
       }
-      if (_FP_strnicmp (line, "Content-Type", 12) == 0)
+      if (FP_strnicmp (line, "Content-Type", 12) == 0)
 	hadct = 1;
     }
 
     if (*state == BEGIN) {
       if ((method == UU_ENCODED || method == XX_ENCODED) &&
 	  (strncmp      (line, "begin ",       6) == 0 ||
-	   _FP_strnicmp (line, "<pre>begin ", 11) == 0)) { /* for LYNX */
+	   FP_strnicmp (line, "<pre>begin ", 11) == 0)) { /* for LYNX */
 	*state = DATA;
 	continue;
       }
@@ -1098,10 +1094,10 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
       }
       else if (method == YENC_ENCODED &&
 	       strncmp (line, "=ybegin ", 8) == 0 &&
-	       _FP_strstr (line, " name=") != NULL) {
+	       FP_strstr (line, " name=") != NULL) {
 	*state = DATA;
 
-	if ((ptr = _FP_strstr (line, " size=")) != NULL) {
+	if ((ptr = FP_strstr (line, " size=")) != NULL) {
 	  ptr += 6;
 	  yefilesize = atoi (ptr);
 	}
@@ -1109,17 +1105,17 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
 	  yefilesize = -1;
 	}
 
-	if ((ptr =_FP_strstr (line, " part="))) {
+	if ((ptr =FP_strstr (line, " part="))) {
           int partno = atoi (ptr + 6);
-       
-	  if ((ptr = _FP_strstr (line, " total=")))
+
+	  if ((ptr = FP_strstr (line, " total=")))
             yenotlastpart = atoi (ptr + 7) != partno;
 
-	  if (_FP_fgets (line, 1200 - 5, datain) == NULL) {
+	  if (FP_fgets (line, 1200 - 5, datain) == NULL) {
 	    break;
 	  }
 
-	  if ((ptr = _FP_strstr (line, " end=")) == NULL) {
+	  if ((ptr = FP_strstr (line, " end=")) == NULL) {
 	    break;
 	  }
 
@@ -1131,7 +1127,7 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
       else {
 	continue;
       }
-      
+
       tc = tf = vlc = 0;
       lc[0] = lc[1] = 0;
     }
@@ -1145,14 +1141,22 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
 
     if (*state == DATA && method == YENC_ENCODED &&
 	strncmp (line, "=yend ", 6) == 0) {
-      if ((ptr = _FP_strstr (line, " pcrc32=")) != NULL) {
+      int lastpart = !yenotlastpart && (yepartends == 0 || yepartends >= yefilesize);
+      yefilecrc = uu_crc32_combine(yefilecrc, yepartcrc, yepartsize);
+      if ((ptr = FP_strstr (line, " pcrc32=")) != NULL) {
 	crc32_t pcrc32 = strtoul (ptr + 8, NULL, 16);
 	if (pcrc32 != yepartcrc) {
 	  UUMessage (uunconc_id, __LINE__, UUMSG_WARNING,
 		     uustring (S_PCRC_MISMATCH), progress.curfile, progress.partno);
 	}
+      } else if ((ptr = FP_strstr (line, " pcrc=")) != NULL) {
+	crc32_t pcrc32 = strtoul (ptr + 6, NULL, 16);
+	if (pcrc32 != yepartcrc) {
+	  UUMessage (uunconc_id, __LINE__, UUMSG_WARNING,
+		     uustring (S_PCRC_MISMATCH), progress.curfile, progress.partno);
+	}
       }
-      if ((ptr = _FP_strstr (line, " crc32=")) != NULL)
+      if (lastpart && (ptr = FP_strstr (line, " crc32=")) != NULL)
       {
 	crc32_t fcrc32 = strtoul (ptr + 7, NULL, 16);
 	if (fcrc32 != yefilecrc) {
@@ -1160,7 +1164,7 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
 		     uustring (S_CRC_MISMATCH), progress.curfile);
 	}
       }
-      if ((ptr = _FP_strstr (line, " size=")) != NULL)
+      if ((ptr = FP_strstr (line, " size=")) != NULL)
       {
 	size_t size = atol(ptr + 6);
 	if (size != yepartsize && yefilesize != -1) {
@@ -1174,7 +1178,7 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
 		       yepartsize, size);
 	}
       }
-      if (!yenotlastpart && (yepartends == 0 || yepartends >= yefilesize)) {
+      if (lastpart) {
 	*state = DONE;
       }
       break;
@@ -1202,8 +1206,7 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
 	if (tf) {
 	  count  = UUDecodeLine (line, oline, method);
 	  if (method == YENC_ENCODED) {
-	    yepartcrc = crc32(yepartcrc, oline, count);
-	    yefilecrc = crc32(yefilecrc, oline, count);
+	    yepartcrc = uu_crc32(yepartcrc, oline, count);
 	    yepartsize += count;
 	  }
 	  vlc++; lc[1]++;
@@ -1231,7 +1234,7 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
 	  lc[1] = 3;
 	}
 	else {
-	  _FP_strncpy (save[tc++], line, 1200);
+	  FP_strncpy (save[tc++], line, 1200);
 	}
 
 	if (method == UU_ENCODED)
@@ -1276,7 +1279,7 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
   if (*state  == DONE ||
       (*state == DATA && method == B64ENCODED &&
        vflag == B64ENCODED && (flags&FL_PROPER || haddh))) {
-    for (tf=0; tf<tc; tf++) 
+    for (tf=0; tf<tc; tf++)
       count += UUDecodeLine (save[tf], oline + count, method);
     if (count) {
       if (method == BH_ENCODED) {
@@ -1348,7 +1351,7 @@ UUDecode (uulist *data)
 
   if (!data->binfile) {
 #else
-  if ((data->binfile = tempnam (NULL, "uu")) == NULL) {
+  if ((data->binfile = tmpnam (NULL)) == NULL) {
 #endif /* HAVE_MKSTEMP */
     UUMessage (uunconc_id, __LINE__, UUMSG_ERROR,
 	       uustring (S_NO_TEMP_NAME));
@@ -1360,7 +1363,7 @@ UUDecode (uulist *data)
   strcat(data->binfile, "/");
   strcat(data->binfile, tmpprefix);
 
-  if ((tmpfd = mkstemp(data->binfile)) == -1 || 
+  if ((tmpfd = mkstemp(data->binfile)) == -1 ||
 	  (dataout = fdopen(tmpfd, mode)) == NULL) {
 #else
   if ((dataout = fopen (data->binfile, mode)) == NULL) {
@@ -1378,12 +1381,13 @@ UUDecode (uulist *data)
 		close(tmpfd);
     }
 #endif /* HAVE_MKSTEMP */
-    _FP_free (data->binfile);
+    FP_free (data->binfile);
     data->binfile = NULL;
     uu_errno = errno;
     return UURET_IOERR;
   }
   UUSETBUF (dataout, dataout_buf, uu_wbuf);
+  FP_flockfile (dataout);
 
   /*
    * we don't have begin lines in Base64 or plain text files.
@@ -1409,13 +1413,13 @@ UUDecode (uulist *data)
    */
   progress.action = 0;
   if (data->filename != NULL) {
-    _FP_strncpy (progress.curfile,
+    FP_strncpy (progress.curfile,
 		 (strlen(data->filename)>255)?
 		 (data->filename+strlen(data->filename)-255):data->filename,
 		 256);
   }
   else {
-    _FP_strncpy (progress.curfile,
+    FP_strncpy (progress.curfile,
 		 (strlen(data->binfile)>255)?
 		 (data->binfile+strlen(data->binfile)-255):data->binfile,
 		 256);
@@ -1431,7 +1435,7 @@ UUDecode (uulist *data)
     progress.numparts = (iter->partno)?iter->partno:1;
     iter = iter->NEXT;
   }
-  
+
   /*
    * let's rock!
    */
@@ -1474,9 +1478,10 @@ UUDecode (uulist *data)
 	res = UURET_IOERR;
 	break;
       }
-      _FP_strncpy (uugen_fnbuffer, iter->data->sfname, 1024);
+      FP_strncpy (uugen_fnbuffer, iter->data->sfname, 1024);
     }
     UUSETBUF (datain, datain_buf, uu_rbuf);
+    FP_flockfile (datain);
 
     progress.partno  = part;
     progress.fsize   = (iter->data->length)?iter->data->length:-1;
@@ -1499,7 +1504,7 @@ UUDecode (uulist *data)
     iter = iter->NEXT;
   }
 
-  if (state == DATA && 
+  if (state == DATA &&
       (data->uudet == B64ENCODED || data->uudet == QP_ENCODED ||
        data->uudet == PT_ENCODED))
     state = DONE; /* assume we're done */
@@ -1514,7 +1519,7 @@ UUDecode (uulist *data)
 
   if (res != UURET_OK || (state != DONE && !uu_desperate)) {
     unlink (data->binfile);
-    _FP_free (data->binfile);
+    FP_free (data->binfile);
     data->binfile = NULL;
     data->state  &= ~UUFILE_TMPFILE;
     data->state  |=  UUFILE_ERROR;
@@ -1537,12 +1542,11 @@ UUDecode (uulist *data)
 
   if (data->uudet == BH_ENCODED && data->binfile) {
 #ifdef HAVE_MKSTEMP
-	  ntmp = malloc(strlen(tmpdir)+strlen(tmpprefix)+2);
-	  
-	  if (ntmp == NULL) {
+    ntmp = malloc(strlen(tmpdir)+strlen(tmpprefix)+2);
 #else
-    if ((ntmp = tempnam (NULL, "uu")) == NULL) {
+    ntmp = tempnam (NULL);
 #endif /* HAVE_MKSTEMP */
+    if (ntmp == NULL) {
       UUMessage (uunconc_id, __LINE__, UUMSG_ERROR,
 		 uustring (S_NO_TEMP_NAME));
       progress.action = 0;
@@ -1557,11 +1561,12 @@ UUDecode (uulist *data)
       return UURET_IOERR;
     }
     UUSETBUF (datain, datain_buf, uu_rbuf);
+    FP_flockfile (datain);
 
 #ifdef HAVE_MKSTEMP
     strcpy(ntmp, tmpdir);
     strcat(ntmp, "/");
-    strcat(ntmp, tmpprefix); 
+    strcat(ntmp, tmpprefix);
     if ((tmpfd = mkstemp(ntmp)) == -1 ||
 		(dataout = fdopen(tmpfd, "wb")) == NULL) {
 #else
@@ -1583,11 +1588,12 @@ UUDecode (uulist *data)
       return UURET_IOERR;
     }
     UUSETBUF (dataout, dataout_buf, uu_wbuf);
+    FP_flockfile (dataout);
 
     /*
      * read fork lengths. remember they're in Motorola format
      */
-    r[0] = _FP_fgetc (datain);
+    r[0] = FP_getc (datain);
     hb   = (int) r[0] + 22;
     fseek (datain, (int) r[0] + 12, SEEK_SET);
     fread (r, 1, 8, datain);
@@ -1633,7 +1639,7 @@ UUDecode (uulist *data)
      * copy the chosen fork
      */
 
-    while (!feof (datain) && numbytes) {
+    while (!FP_feof (datain) && numbytes) {
       if (UUBUSYPOLL(ftell(datain)-progress.foffset,progress.fsize)) {
 	UUMessage (uunconc_id, __LINE__, UUMSG_NOTE,
 		   uustring (S_DECODE_CANCEL));
@@ -1649,7 +1655,7 @@ UUDecode (uulist *data)
       bytes = fread (uugen_inbuffer, 1,
 		     (size_t) ((numbytes>1024)?1024:numbytes), datain);
 
-      if (ferror (datain) || (bytes == 0 && !feof (datain))) {
+      if (ferror (datain) || (bytes == 0 && !FP_feof (datain))) {
 	progress.action = 0;
 	UUMessage (uunconc_id, __LINE__, UUMSG_ERROR,
 		   uustring (S_SOURCE_READ_ERR),
@@ -1735,13 +1741,13 @@ UUQuickDecode (FILE *datain, FILE *dataout, char *boundary, long maxpos)
   memset (&myenv, 0, sizeof (headers));
   UUScanHeader (datain, &myenv);
 
-  if (_FP_stristr (myenv.ctenc, "uu") != NULL)
+  if (FP_stristr (myenv.ctenc, "uu") != NULL)
     encoding = UU_ENCODED;
-  else if (_FP_stristr (myenv.ctenc, "xx") != NULL)
+  else if (FP_stristr (myenv.ctenc, "xx") != NULL)
     encoding = XX_ENCODED;
-  else if (_FP_stricmp (myenv.ctenc, "base64") == 0)
+  else if (FP_stricmp (myenv.ctenc, "base64") == 0)
     encoding = B64ENCODED;
-  else if (_FP_stricmp (myenv.ctenc, "quoted-printable") == 0)
+  else if (FP_stricmp (myenv.ctenc, "quoted-printable") == 0)
     encoding = QP_ENCODED;
   else
     encoding = PT_ENCODED;
