@@ -1,7 +1,7 @@
 /*
  * This file is part of uudeview, the simple and friendly multi-part multi-
- * file uudecoder  program  (c)  1994 by Frank Pilhofer. The author may be
- * contacted by his email address,          fp@informatik.uni-frankfurt.de
+ * file uudecoder  program  (c) 1994-2001 by Frank Pilhofer. The author may
+ * be contacted at fp@fpx.de
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -49,12 +49,12 @@
 #include <errno.h>
 #endif
 
-#include <uulib.h>
+#include <uudeview.h>
 #include <uuint.h>
 #include <fptools.h>
 #include <uustring.h>
 
-char * uuutil_id = "$Id: uuutil.c,v 1.13 1996/08/21 19:55:16 fp Exp $";
+char * uuutil_id = "$Id$";
 
 /*
  * Parts with different known extensions will not be merged by SPMS.
@@ -215,7 +215,7 @@ UUSMPKnownExt (char *filename)
  */
 
 int
-UUbhdecomp (char *in, char *out, char *last, int *rpc, 
+UUbhdecomp (char *in, char *out, char *last, int *rpc,
 	    size_t inc, size_t max, size_t *opc)
 {
   size_t count, used=0, dummy;
@@ -435,38 +435,41 @@ UUSmerge (int pass)
   uulist *iter = UUGlobalFileList, *last=NULL, *res, *temp;
   int flag = 0;
 
-  while (iter) {
-    if ((iter->state & UUFILE_OK) || iter->uudet == 0) {
-      last = iter;
-      iter = iter->NEXT;
-      continue;
-    }
-    if ((res = UU_smparts_r (iter, pass)) != NULL) {
-      UUMessage (uuutil_id, __LINE__, UUMSG_MESSAGE,
-		 uustring (S_SMERGE_MERGED),
-		 (iter->subfname) ? iter->subfname : "",
-		 (res->subfname)  ? res->subfname  : "", pass);
- 
-      temp       = iter->NEXT;
-      iter->NEXT = NULL;
-      UUkilllist (iter);
+  if (pass >= 0)
+    {
+      while (iter) {
+        if ((iter->state & UUFILE_OK) || iter->uudet == 0) {
+          last = iter;
+          iter = iter->NEXT;
+          continue;
+        }
+        if ((res = UU_smparts_r (iter, pass)) != NULL) {
+          UUMessage (uuutil_id, __LINE__, UUMSG_MESSAGE,
+		     uustring (S_SMERGE_MERGED),
+		     (iter->subfname) ? iter->subfname : "",
+		     (res->subfname)  ? res->subfname  : "", pass);
 
-      flag++;
+          temp       = iter->NEXT;
+          iter->NEXT = NULL;
+          UUkilllist (iter);
 
-      if (last == NULL) {
-	UUGlobalFileList = temp;
-	iter             = temp;
+          flag++;
+
+          if (last == NULL) {
+	    UUGlobalFileList = temp;
+	    iter             = temp;
+          }
+          else {
+	    last->NEXT       = temp;
+	    iter             = temp;
+          }
+
+          continue;
+        }
+        last = iter;
+        iter = iter->NEXT;
       }
-      else {
-	last->NEXT       = temp;
-	iter             = temp;
-      }
-
-      continue;
     }
-    last = iter;
-    iter = iter->NEXT;
-  }
 
   /*
    * check again
@@ -477,9 +480,3 @@ UUSmerge (int pass)
   return flag;
 }
 
-
-/*****************************************************************************
- + Frank Pilhofer                             fp@informatik.uni-frankfurt.de +
- +---------------------------------------------------------------------------+
- | Department of Computer Sciences * University of Frankfurt / Main, Germany |
- *****************************************************************************/
